"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _schemaError = require("./schema-error");

var _schemaError2 = _interopRequireDefault(_schemaError);

var _verifyTreeIsWalkable = require("./verify-tree-is-walkable");

var _walkTree = require("./walk-tree");

var walkTypeTree = _interopRequireWildcard(_walkTree);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function checkForReferenceCycle(types) {
  function followReferences(tree, types, visited) {
    if (tree.name !== "reference") return;
    if (visited.indexOf(tree.referenceName) !== -1) return visited.concat(tree.name);
    return followReferences(types[tree.referenceName], types, visited.concat(tree.referenceName));
  }

  Object.keys(types).forEach(function (typeName) {
    var typeCycle = followReferences(types[typeName], types, [typeName]);
    if (typeCycle) throw new _schemaError2.default("Found a reference cycle consisting of types: [" + typeCycle.join(", ") + "]", typeName);
  });
}

function checkReferenceValidities(types) {
  Object.keys(types).forEach(function (typeName) {
    walkTypeTree.forEach(types[typeName], function (_ref) {
      var name = _ref.name,
          referenceName = _ref.referenceName;

      if (name !== "reference") return;

      if (!types[referenceName]) throw new _schemaError2.default("Found a reference to the type \"" + referenceName + "\", but no definition for that type", typeName);
    });
  });
}

function checkThatEntriesAreTrees(types) {
  Object.keys(types).forEach(function (typeName) {
    var tree = types[typeName];

    if (typeof tree === "function") throw new _schemaError2.default("Found a function in the type graph. Did you forget to call the creator for a primitive, " + "like passing 'string' instead of 'string()'?", typeName);

    (0, _verifyTreeIsWalkable.verifyTreeIsWalkable)(tree, typeName);
  });
}

exports.default = function (types) {
  checkThatEntriesAreTrees(types);
  checkReferenceValidities(types);
  checkForReferenceCycle(types);
};