"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.compileDocumentation = exports.compileValidators = undefined;

var _compiler = require("./compiler");

var _compiler2 = _interopRequireDefault(_compiler);

var _validatorInterpreters = require("./validator-interpreters");

var _validatorInterpreters2 = _interopRequireDefault(_validatorInterpreters);

var _documentationInterpreters = require("./documentation-interpreters");

var _documentationInterpreters2 = _interopRequireDefault(_documentationInterpreters);

var _utils = require("./utils");

var _schemaError = require("./schema-error");

var _schemaError2 = _interopRequireDefault(_schemaError);

var _checkTypeGraph = require("./check-type-graph");

var _checkTypeGraph2 = _interopRequireDefault(_checkTypeGraph);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function wrapInErrorHandler(fn) {
  return function schemaErrorHandler() {
    try {
      return fn.apply(undefined, arguments);
    } catch (e) {
      if (e instanceof _schemaError2.default) {
        throw new Error(e.errorMessage);
      }
      throw e;
    }
  };
}

var compileValidators = wrapInErrorHandler(function (scope) {
  var customInterpreters = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var types = scope.getTypes();

  (0, _checkTypeGraph2.default)(types);

  Object.keys(types).forEach(function (typeName) {
    if (_validatorInterpreters2.default[typeName]) throw new _schemaError2.default("Cannot redefine built-in type " + typeName, typeName);
  });

  var withTypeNames = (0, _utils.mapObject)(types, function (value, key) {
    var cloned = (0, _utils.cloneDeep)(value);
    cloned.meta.typeName = key;
    return cloned;
  });

  var customWithNiceErrorMessages = (0, _utils.mapObject)(customInterpreters, function (interpreter, typeName) {
    return function (tree, recurse, compiled) {
      var bare = interpreter(tree, recurse, compiled),
          name = tree.meta.typeName;
      return function (x) {
        var err = bare(x);
        if (err) return formatError(err, x, name);
      };
    };
  });

  var compiled = (0, _compiler2.default)(withTypeNames, _validatorInterpreters2.default, customWithNiceErrorMessages);

  return (0, _utils.mapObject)(compiled, function (fn) {
    return function (x) {
      var err = fn(x);return err && err.message;
    };
  });
});

function formatError(_ref, found, name) {
  var message = _ref.message,
      innerError = _ref.innerError;

  return {
    found: found,
    message: "In type \"" + name + "\", " + message + (innerError ? ":\n" + innerError.message : "")
  };
}

var compileDocumentation = wrapInErrorHandler(function (scope) {
  var customInterpreters = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var types = scope.getTypes();
  (0, _checkTypeGraph2.default)(types);

  return (0, _compiler2.default)(types, _documentationInterpreters2.default, customInterpreters);
});

exports.compileValidators = compileValidators;
exports.compileDocumentation = compileDocumentation;