"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (typeDefinitions, interpreters, customInterpreters) {

  var allInterpreters = (0, _utils.mergeObjects)(interpreters, { custom: customInterpreters });

  var compiledDefinitions = {};

  Object.keys(typeDefinitions).forEach(function (typeName) {
    compiledDefinitions[typeName] = compile(typeDefinitions[typeName], typeDefinitions, compiledDefinitions, allInterpreters);
  });

  return compiledDefinitions;
};

var _utils = require("./utils");

var _schemaError = require("./schema-error");

var _schemaError2 = _interopRequireDefault(_schemaError);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var markAsCompiled = function markAsCompiled(compiledValue) {
  return { name: "compiled", compiledValue: compiledValue };
};

function compile(tree, typeDefinitions, compiledTypes, interpreters) {

  var name = tree.name;

  if (name === "compiled") return tree.compiledValue;

  if (name === "reference") {
    if (!typeDefinitions[tree.referenceName]) throw new _schemaError2.default("Found a forward reference to the type \"" + reference + "\" but no definition for that type");

    tree.getCompiledTarget = function () {
      return compiledTypes[tree.referenceName];
    };

    return interpreters.reference(tree);
  }

  var recurse = function recurse(subtree) {
    return compile(subtree, typeDefinitions, compiledTypes, interpreters);
  };

  if (name === "custom") {
    var customInterpreter = interpreters.custom[tree.label];
    if (!customInterpreter) throw new _schemaError2.default("Found a reference to the custom type " + tree.label + " but no custom interpreter for this type");

    return customInterpreter(tree, recurse, markAsCompiled);
  }

  var interpreter = interpreters[name];

  if (!interpreter) throw new _schemaError2.default("Found a reference to the type " + name + " but no interpreter for this type");

  return interpreter(tree, recurse, markAsCompiled);
}