"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.TypeNode = exports.schema = exports.core = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _utils = require("./utils");

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var TypeNode = function TypeNode() {
  _classCallCheck(this, TypeNode);
};

function makeNodeFactory(_ref, name) {
  var plainFields = _ref.plainFields,
      childFields = _ref.childFields,
      objectFields = _ref.objectFields,
      listField = _ref.listField;

  var argList = (plainFields || []).concat(childFields || []).concat(objectFields || []);

  return function () {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    var node = new TypeNode();
    node.name = name;
    node.meta = {};

    var plainArgs = args.slice(0, argList.length),
        listArgs = args.slice(argList.length);

    (0, _utils.zip)(argList, plainArgs).forEach(function (_ref2) {
      var _ref3 = _slicedToArray(_ref2, 2),
          fieldName = _ref3[0],
          value = _ref3[1];

      return node[fieldName] = value;
    });

    if (listField) node[listField] = listArgs;

    return node;
  };
}

var schema = {
  string: {},
  boolean: {},
  number: {},
  function: {},
  object: {},
  any: {},
  literal: {
    plainFields: ["value"]
  },
  array: {
    childFields: ["contents"]
  },
  laxStruct: {
    objectFields: ["fields"]
  },
  strictStruct: {
    objectFields: ["fields"]
  },
  dictionary: {
    childFields: ["keys", "values"]
  },
  optional: {
    childFields: ["contents"]
  },
  reference: {
    plainFields: ["referenceName"]
  },
  custom: {
    plainFields: ["label"],
    listField: "args"
  },
  intersection: {
    listField: "parents"
  },
  alternatives: {
    listField: "options"
  }
};

var core = (0, _utils.mapObject)(schema, makeNodeFactory);

core.enum = function () {
  for (var _len2 = arguments.length, members = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    members[_key2] = arguments[_key2];
  }

  return core.alternatives.apply(core, _toConsumableArray(members.map(core.literal)));
};

exports.core = core;
exports.schema = schema;
exports.TypeNode = TypeNode;