"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

function struct(_ref, recurse) {
  var fields = _ref.fields;

  var fieldList = Object.keys(fields).map(function (fieldName) {
    return "  " + fieldName + ": " + recurse(fields[fieldName]);
  });

  return "{\n" + fieldList.join("\n") + "\n}";
}

var interpreters = {
  laxStruct: function laxStruct(tree, recurse) {
    return struct(tree, recurse) + " // May contain additional fields";
  },
  strictStruct: struct,
  literal: function literal(_ref2) {
    var value = _ref2.value;
    return JSON.stringify(value);
  },
  optional: function optional(_ref3, recurse) {
    var contents = _ref3.contents;
    return "optional(" + recurse(contents) + ")";
  },
  alternatives: function alternatives(_ref4, recurse) {
    var options = _ref4.options;
    return options.map(recurse).join(" | ");
  },
  dictionary: function dictionary(_ref5, recurse) {
    var keys = _ref5.keys,
        values = _ref5.values;
    return "Dictionary<" + recurse(keys) + ", " + recurse(values) + ">";
  },
  array: function array(_ref6, recurse) {
    var contents = _ref6.contents;
    return "[" + recurse(contents) + "]";
  },
  intersection: function intersection(_ref7, recurse) {
    var parents = _ref7.parents;
    return parents.map(recurse).join(" & ");
  },
  reference: function reference(_ref8) {
    var referenceName = _ref8.referenceName;
    return referenceName;
  },
  string: function string() {
    return "string";
  },
  number: function number() {
    return "number";
  },
  boolean: function boolean() {
    return "boolean";
  },
  function: function _function() {
    return "function";
  },
  object: function object() {
    return "object";
  },
  any: function any() {
    return "any";
  }
};

exports.default = interpreters;