"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

function oneArgument(fn) {
  return function (x) {
    return fn(x);
  };
}

function mapObject(obj, fn) {
  return Object.keys(obj).map(function (key) {
    return { key: key, value: fn(obj[key], key) };
  }).reduce(function (acc, _ref) {
    var key = _ref.key,
        value = _ref.value;
    acc[key] = value;return acc;
  }, {});
}

function mergeObjects() {
  for (var _len = arguments.length, objs = Array(_len), _key = 0; _key < _len; _key++) {
    objs[_key] = arguments[_key];
  }

  return objs.reduce(function (acc, obj) {
    Object.keys(obj).forEach(function (key) {
      acc[key] = obj[key];
    });
    return acc;
  }, {});
}

function zip(arr1, arr2) {
  var result = [];

  for (var i = 0; i < arr1.length && i < arr2.length; i++) {
    result[i] = [arr1[i], arr2[i]];
  }

  return result;
}

function firstTruthy(array, mapper) {
  for (var i = 0; i < array.length; i++) {
    var result = mapper(array[i]);
    if (result) return result;
  }
}

function anyPassing(array, predicate) {
  for (var i = 0; i < array.length; i++) {
    var result = predicate(array[i]);
    if (result) return true;
  }
  return false;
}

function firstTruthyWithIndex(array, mapper) {
  for (var i = 0; i < array.length; i++) {
    var result = mapper(array[i]);
    if (result) return [result, i];
  }

  return [];
}

function arrayDifference(arr1, arr2) {
  return arr1.filter(function (x) {
    return arr2.indexOf(x) === -1;
  }).concat(arr2.filter(function (x) {
    return arr1.indexOf(x) === -1;
  }));
}

function arrayIntersection(arr1, arr2) {
  return arr1.filter(function (x) {
    return arr2.indexOf(x) !== -1;
  });
}

function clone(obj) {
  if ((typeof obj === "undefined" ? "undefined" : _typeof(obj)) === "object") return mapObject(obj, function (x) {
    return x;
  });

  return obj;
}

function cloneDeep(obj) {
  var seen = [],
      created = [];

  function recursive(x) {
    var visited = seen.indexOf(x);
    if (visited !== -1) return created[visited];

    if (Array.isArray(x)) {
      var newArray = [];

      seen.push(x);
      created.push(newArray);

      return x.map(recursive);
    } else if ((typeof x === "undefined" ? "undefined" : _typeof(x)) === "object") {
      var _ret = function () {

        if (x === null) return {
            v: null
          };

        var newObj = {};

        seen.push(x);
        created.push(newObj);

        Object.keys(x).forEach(function (key) {
          newObj[key] = recursive(x[key]);
        });

        return {
          v: newObj
        };
      }();

      if ((typeof _ret === "undefined" ? "undefined" : _typeof(_ret)) === "object") return _ret.v;
    }

    return x;
  }

  return recursive(obj);
}

exports.oneArgument = oneArgument;
exports.mapObject = mapObject;
exports.mergeObjects = mergeObjects;
exports.firstTruthy = firstTruthy;
exports.anyPassing = anyPassing;
exports.firstTruthyWithIndex = firstTruthyWithIndex;
exports.arrayDifference = arrayDifference;
exports.clone = clone;
exports.cloneDeep = cloneDeep;
exports.arrayIntersection = arrayIntersection;
exports.zip = zip;