"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _utils = require("./utils");

var _coreCombinators = require("./core-combinators");

function addToError(found, original, message) {
  return {
    found: found,
    message: message + (original.message ? "\n" + original.message : "")
  };
}

function laxStruct(_ref, recurse, compiled) {
  var fields = _ref.fields,
      meta = _ref.meta;

  var compiledFields = (0, _utils.mapObject)(fields, recurse);

  var descriptor = _coreCombinators.core.intersection(_coreCombinators.core.object(), compiled(function (x) {
    var _ref2 = (0, _utils.firstTruthy)(Object.keys(compiledFields), function (fieldName) {
      var error = compiledFields[fieldName](x[fieldName]);
      return error ? [fieldName, error] : null;
    }) || [],
        _ref3 = _slicedToArray(_ref2, 2),
        badField = _ref3[0],
        err = _ref3[1];

    if (err) return meta.silent ? err : err.found === undefined ? { found: x, message: "In type \"" + (meta.typeName || "laxStruct") + "\", expected a field \"" + badField + "\" but it was missing" } : addToError(x, err, "In type \"" + (meta.typeName || "laxStruct") + "\", in field \"" + badField + "\":");
  }));

  descriptor.meta.silent = true;

  return recurse(descriptor);
}

function strictStruct(_ref4, recurse, compiled) {
  var fields = _ref4.fields,
      meta = _ref4.meta;

  var laxStructDescriptor = _coreCombinators.core.laxStruct(fields);
  laxStructDescriptor.meta.typeName = meta.typeName;

  var descriptor = _coreCombinators.core.intersection(laxStructDescriptor, compiled(function (x) {
    var extraKey = Object.keys(x).find(function (key) {
      return !fields[key];
    });
    if (extraKey) return {
      found: x,
      message: "In type \"" + (meta.typeName || "strictStruct") + "\", found an extra key \"" + extraKey + "\".\nAllowed keys: " + Object.keys(fields).map(function (k) {
        return '"' + k + '"';
      }).join(", ") };
  }));

  descriptor.meta.silent = true;

  return recurse(descriptor);
}

function literal(_ref5) {
  var value = _ref5.value;

  return function (x) {
    return x !== value ? {
      found: x,
      message: "Expected the literal value " + JSON.stringify(value) + ", but found " + JSON.stringify(x)
    } : null;
  };
}

function optional(_ref6, recurse) {
  var contents = _ref6.contents,
      meta = _ref6.meta;

  var validateContents = recurse(contents);

  return function (x) {
    var error = validateContents(x);
    if (!error || error.found === null || error.found === undefined) return;
    return addToError(x, error, "In type \"" + (meta.typeName || "Optional") + "\":");
  };
}

function alternatives(_ref7, recurse) {
  var options = _ref7.options,
      meta = _ref7.meta;

  var compiledOptions = options.map(recurse);

  return function (x) {
    for (var i = 0; i < compiledOptions.length; i++) {
      if (!compiledOptions[i](x)) return;
    }
    return (0, _utils.anyPassing)(compiledOptions, function (option) {
      return !option(x);
    }) ? null : { found: x, message: "Could not match any of the alternatives in type \"" + (meta.typeName || "Alternatives") + "\" to the value " + JSON.stringify(x) };
  };
}

function dictionary(_ref8, recurse, compiled) {
  var keys = _ref8.keys,
      values = _ref8.values,
      meta = _ref8.meta;

  var keyValidator = recurse(keys),
      valueValidator = recurse(values);

  var descriptor = _coreCombinators.core.intersection(_coreCombinators.core.object(), compiled(function (x) {

    var keyError = (0, _utils.firstTruthy)(Object.keys(x), function (key) {
      var error = keyValidator(key);
      return error ? error : null;
    });

    if (keyError) return addToError(x, keyError, "In type \"" + (meta.typeName || "Dictionary") + "\", found a key of an unexpected type:");

    var valueError = (0, _utils.firstTruthy)(Object.keys(x), function (key) {
      var error = valueValidator(x[key]);
      return error ? error : null;
    });

    if (valueError) return addToError(x, valueError, "In type \"" + (meta.typeName || "Dictionary") + "\", found a value of an unexpected type:");
  }));

  descriptor.meta.silent = true;

  return recurse(descriptor);
}

function _array(_ref9, recurse) {
  var contents = _ref9.contents,
      meta = _ref9.meta;

  var contentsValidator = recurse(contents);
  return function (x) {
    if (!x || !Array.isArray(x)) return {
      found: x,
      message: "Expected a value of type \"" + meta.typeName + "\" but found: " + JSON.stringify(x)
    };

    var _firstTruthyWithIndex = (0, _utils.firstTruthyWithIndex)(x, contentsValidator),
        _firstTruthyWithIndex2 = _slicedToArray(_firstTruthyWithIndex, 2),
        error = _firstTruthyWithIndex2[0],
        index = _firstTruthyWithIndex2[1];

    if (error) return addToError(x, error, "In type \"" + (meta.typeName || "Array") + "\" at index " + index + ":");
  };
}

function intersection(_ref10, recurse) {
  var parents = _ref10.parents,
      meta = _ref10.meta;

  var validators = parents.map(recurse);

  return function (x) {
    var error = (0, _utils.firstTruthy)(validators, function (validator) {
      return validator(x);
    });
    if (error) return meta.silent ? error : addToError(x, error, "In type \"" + (meta.typeName || "Intersection") + "\":");
  };
}

// We need to evaluate this lazily, because this may be a forward reference,
// meaning that getCompiledTarget may throw an error until compilation is complete
function reference(_ref11) {
  var getCompiledTarget = _ref11.getCompiledTarget;

  return function (x) {
    return getCompiledTarget()(x);
  };
}

function compilePrimitive(tester, typeName) {
  return function (_ref12) {
    var meta = _ref12.meta;
    return function (x) {
      return tester(x) ? null : { found: x, message: "Expected a value of type \"" + (meta.typeName || typeName) + "\" but found: " + JSON.stringify(x) };
    };
  };
}

var primitives = (0, _utils.mapObject)({
  string: function string(x) {
    return typeof x === "string";
  },
  number: function number(x) {
    return !isNaN(x) && typeof x === "number";
  },
  boolean: function boolean(x) {
    return typeof x === "boolean";
  },
  function: function _function(x) {
    return typeof x === "function";
  },
  object: function object(x) {
    return (typeof x === "undefined" ? "undefined" : _typeof(x)) === "object" && !Array.isArray(x) && x !== null;
  },
  any: function any() {
    return true;
  }
}, compilePrimitive);

exports.default = (0, _utils.mergeObjects)(primitives, {
  literal: literal,
  array: _array,
  laxStruct: laxStruct,
  strictStruct: strictStruct,
  dictionary: dictionary,
  optional: optional,
  alternatives: alternatives,
  intersection: intersection,
  reference: reference
});