"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.verifyTreeIsWalkable = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _coreCombinators = require("./core-combinators");

var _utils = require("./utils");

var _schemaError = require("./schema-error");

var _schemaError2 = _interopRequireDefault(_schemaError);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function detectErrorsInNode(node) {
  if (!node || (typeof node === "undefined" ? "undefined" : _typeof(node)) !== "object") return "Found a non-object in the type graph";
  if (!node.name) return "Found a type graph node with no 'name' field";
  if (!node.meta) return "Found a type graph node with no 'meta' field";

  var nodeSchema = _coreCombinators.schema[node.name];

  var fieldError = (0, _utils.firstTruthy)(nodeSchema.plainFields || [], function (fieldName) {
    return node.hasOwnProperty(fieldName) ? null : "Found a " + node.name + " node with no \"" + fieldName + "\" field";
  });

  var childError = (0, _utils.firstTruthy)(nodeSchema.childFields || [], function (fieldName) {
    return node[fieldName] ? null : "Found a " + node.name + " node with no \"" + fieldName + "\" field";
  });

  var listError = nodeSchema.listField ? Array.isArray(node[nodeSchema.listField]) ? null : "Found a " + node.name + " node without an array \"" + nodeSchema.listField + "\" field" : null;

  var objectError = (0, _utils.firstTruthy)(nodeSchema.objectFields || [], function (fieldName) {
    return node[fieldName] && _typeof(node[fieldName]) === "object" ? null : "Found a " + node.name + " node without an array \"" + fieldName + "\" field";
  });

  return fieldError || childError || listError || objectError;
}

function verifyTreeIsWalkable(tree, typeName) {
  var checkRecursively = function checkRecursively(node, path, inCustomType) {

    // Custom types may take arguments other than type nodes, so we filter those out from validation
    if (inCustomType && !(node instanceof _coreCombinators.TypeNode)) return;

    var error = detectErrorsInNode(tree);
    if (error) return { error: error, path: path };

    var nodeSchema = _coreCombinators.schema[node.name];
    if (!nodeSchema) return {
      error: "Found a node with an invalid name: \"" + node.name + "\".\nAllowed node types are: [" + Object.keys(_coreCombinators.schema).join(", ") + "]",
      path: path
    };

    if (nodeSchema.childFields) {
      var childError = (0, _utils.firstTruthy)(nodeSchema.childFields, function (fieldName) {
        return checkRecursively(node[fieldName], path.concat("In a node of type " + node.name + ", in field \"" + fieldName + "\""), node.name === "custom");
      });

      if (childError) return childError;
    }

    if (nodeSchema.objectFields) {
      var objectError = (0, _utils.firstTruthy)(nodeSchema.objectFields, function (fieldName) {
        return (0, _utils.firstTruthy)(Object.keys(node[fieldName]), function (subFieldName) {
          var pathMessage = "In a node of type " + node.name + ", in field \"" + fieldName + "." + subFieldName + "\"";
          var subField = node[fieldName][subFieldName];
          return checkRecursively(subField, path.concat(pathMessage), node.name === "custom");
        });
      });

      if (objectError) return objectError;
    }

    if (nodeSchema.listField) {
      var _ret = function () {
        var fieldName = nodeSchema.listField,
            list = node[fieldName],
            listError = list.map(function (item, index) {
          return checkRecursively(item, path.concat("In a node of type " + node.name + ", in field \"" + fieldName + "\", at index " + index), node.name === "custom");
        }).filter(Boolean)[0];

        if (listError) return {
            v: listError
          };
      }();

      if ((typeof _ret === "undefined" ? "undefined" : _typeof(_ret)) === "object") return _ret.v;
    }
  };

  var error = checkRecursively(tree, [], false);
  if (error) {
    var errorMessage = error.path.join("\n") + "\n" + error.error;
    throw new _schemaError2.default(errorMessage, typeName);
  }
}

exports.verifyTreeIsWalkable = verifyTreeIsWalkable;