"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.forEach = undefined;

var _coreCombinators = require("./core-combinators");

var _utils = require("./utils");

function makeNodeTraverser(_ref) {
  var childFields = _ref.childFields,
      objectFields = _ref.objectFields,
      listField = _ref.listField;

  var fns = [];

  if (childFields) fns.push(function (tree, fn) {
    return childFields.forEach(function (fieldName) {
      return fn(tree[fieldName]);
    });
  });
  if (objectFields) fns.push(function (tree, fn) {
    return objectFields.forEach(function (fieldName) {
      return (0, _utils.mapObject)(tree[fieldName], fn);
    });
  });
  if (listField) fns.push(function (tree, fn) {
    return tree[listField].forEach(fn);
  });

  return function (tree, fn) {
    fns.forEach(function (traversalFunction) {
      return traversalFunction(tree, fn);
    });
  };
}

var applyFunctionToChildren = (0, _utils.mapObject)(_coreCombinators.schema, makeNodeTraverser);

function forEach(tree, fn) {
  fn(tree);
  applyFunctionToChildren[tree.name](tree, fn);
}

exports.forEach = forEach;